const { pop } = require('../uuids')
const { channelEvents } = require('./channels')
const crud = require('./crud')

const connections = {};

const handler = (ws, id) => {
  connections[id] = {
    connection: ws,
    events: [],
  }
  
  ws.send(JSON.stringify({
    event: 'info',
    data: 'Welcome to APL Nuke v1.0.0!'
  }))

  ws.on('message', (msg) => handleMsg(msg, id))
  ws.on('close', () => {
    delete connections[id]
    pop(id)
  })
}

const handleMsg = async (msg, id) => {
  try {
    const d = JSON.parse(msg)
    if (d.hasOwnProperty('subscribe') && channelEvents.indexOf(d.subscribe) !== -1) {
      const channel = d.subscribe.split(':')[0]
      connections[id].events.push(d.subscribe)
      connections[id].connection.send(JSON.stringify(await crud[channel].getAll()))
    } else if (d.hasOwnProperty('event') && channelEvents.indexOf(d.event) !== -1) {
      const ev = d.event.split(':')
      const channel = ev[0]
      const event = ev[1]
      console.log(`received event for ${channel} with data %s`, d.data)
      switch (event) {
        case 'create':
          await crud[channel].create(d.data)
          fanoutMsg(channel, await crud[channel].getAll())
          break;
        case 'update':
          await crud[channel].update(d.data.id, d.data.data)
          fanoutMsg(channel, await crud[channel].getAll())
          break;
          case 'delete':
            await crud[channel].delete(d.data.id)
            fanoutMsg(channel, await crud[channel].getAll())
            break;
      }
    }
  } catch (e) {
    console.log(e)
  }
}

const fanoutMsg = (channel, data) => {
  Object.keys(connections).forEach((k) => {
    if (connections[k].events.indexOf(`${channel}:read`) !== -1) {
      connections[k].connection.send(JSON.stringify(data))
    }
  })
}

module.exports = handler